@@ -239,6 +239,10 @@ int hcid_dbus_request_pin(int dev, bdaddr_t *sba, struct hci_conn_info *ci)
 		return -1;
 	}
 
+	if (!adapter_pairing_initiator(adapter, &ci->bdaddr) &&
+			!adapter_is_pairable(adapter))
+		return -EPERM;
+
 	ba2str(&ci->bdaddr, addr);
 
 	device = adapter_find_device(adapter, addr);
@@ -1244,44 +1248,6 @@ void hcid_dbus_disconn_complete(bdaddr_t *local, uint8_t status,
 	}
 }
 
-int set_limited_discoverable(int dd, const uint8_t *cls, gboolean limited)
-{
-	uint32_t dev_class;
-	int num = (limited ? 2 : 1);
-	uint8_t lap[] = { 0x33, 0x8b, 0x9e, 0x00, 0x8b, 0x9e };
-	/*
-	 * 1: giac
-	 * 2: giac + liac
-	 */
-	if (hci_write_current_iac_lap(dd, num, lap, HCI_REQ_TIMEOUT) < 0) {
-		int err = errno;
-		error("Can't write current IAC LAP: %s(%d)",
-				strerror(err), err);
-		return -err;
-	}
-
-	if (limited) {
-		if (cls[1] & 0x20)
-			return 0; /* Already limited */
-
-		dev_class = (cls[2] << 16) | ((cls[1] | 0x20) << 8) | cls[0];
-	} else {
-		if (!(cls[1] & 0x20))
-			return 0; /* Already clear */
-
-		dev_class = (cls[2] << 16) | ((cls[1] & 0xdf) << 8) | cls[0];
-	}
-
-	if (hci_write_class_of_dev(dd, dev_class, HCI_REQ_TIMEOUT) < 0) {
-		int err = errno;
-		error("Can't write class of device: %s (%d)",
-							strerror(err), err);
-		return -err;
-	}
-
-	return 0;
-}
-
 int set_service_classes(int dd, const uint8_t *cls, uint8_t value)
 {
 	uint32_t dev_class;
@@ -1292,43 +1258,10 @@ int set_service_classes(int dd, const uint8_t *cls, uint8_t value)
 	dev_class = (value << 16) | (cls[1] << 8) | cls[0];
 
 	if (hci_write_class_of_dev(dd, dev_class, HCI_REQ_TIMEOUT) < 0) {
-		int err = errno;
+		int err = -errno;
 		error("Can't write class of device: %s (%d)",
 							strerror(err), err);
-		return -err;
-	}
-
-	return 0;
-}
-
-int set_major_class(int dd, const uint8_t *cls, uint8_t major)
-{
-	uint32_t dev_class;
-
-	dev_class = (cls[2] << 16) | ((cls[1] & 0x20) << 8) |
-						((major & 0xdf) << 8) | 0x00;
-
-	if (hci_write_class_of_dev(dd, dev_class, HCI_REQ_TIMEOUT) < 0) {
-		int err = errno;
-		error("Can't write class of device: %s (%d)",
-							strerror(err), err);
-		return -err;
-	}
-
-	return 0;
-}
-
-int set_minor_class(int dd, const uint8_t *cls, uint8_t minor)
-{
-	uint32_t dev_class;
-
-	dev_class = (cls[2] << 16) | (cls[1] << 8) | minor;
-
-	if (hci_write_class_of_dev(dd, dev_class, HCI_REQ_TIMEOUT) < 0) {
-		int err = errno;
-		error("Can't write class of device: %s (%d)",
-							strerror(err), err);
-		return -err;
+		return err;
 	}
 
 	return 0;
@@ -1343,10 +1276,10 @@ int set_major_and_minor_class(int dd, const uint8_t *cls,
 						((major & 0xdf) << 8) | minor;
 
 	if (hci_write_class_of_dev(dd, dev_class, HCI_REQ_TIMEOUT) < 0) {
-		int err = errno;
+		int err = -errno;
 		error("Can't write class of device: %s (%d)",
 							strerror(err), err);
-		return -err;
+		return err;
 	}
 
 	return 0;
@@ -1440,10 +1373,7 @@ void hcid_dbus_setscan_enable_complete(bdaddr_t *local)
 		goto failed;
 	}
 
-	adapter_remove_discov_timeout(adapter);
-
-	if (adapter_get_scan_mode(adapter) != rp.enable)
-		adapter_mode_changed(adapter, rp.enable);
+	adapter_mode_changed(adapter, rp.enable);
 
 failed:
 	if (dd >= 0)
@@ -1537,6 +1467,10 @@ int hcid_dbus_get_io_cap(bdaddr_t *local, bdaddr_t *remote,
 	if (get_auth_requirements(local, remote, auth) < 0)
 		return -1;
 
+	if (!adapter_pairing_initiator(adapter, remote) &&
+			!adapter_is_pairable(adapter))
+		return -EPERM;
+
 	ba2str(remote, addr);
 
 	/* For CreatePairedDevice use dedicated bonding */
